/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "AlienCommandDialog.h"

#include "AlienCommandModel.h"

#include "ColumnSelectionMenu.h"
#include "ColumnSortingMenu.h"
#include "ContextMenu.h"
#include "IconSize.h"
#include "TreeView.h"

#include <QHeaderView>
#include <QShortcut>

namespace Alien
{

        //__________________________________________________
    CommandDialog::CommandDialog( QWidget* parent ):
        CustomDialog( parent, CloseButton )
    {

        Debug::Throw( "Alien::CommandDialog::CommandDialog.\n" );
        setWindowTitle( tr( "Commands" ) );

        // customize layout
        layout()->setMargin(0);
        layout()->setSpacing(0);
        buttonLayout().setMargin(5);

        setAttribute( Qt::WA_DeleteOnClose );
        setOptionName( "ALIEN_COMMAND_DIALOG" );
        mainLayout().addWidget( list_ = new TreeView(this) );

        list_->header()->hide();
        list_->setSortingEnabled( true );
        list_->setIconSize( IconSize::get( IconSize::Medium ) );
        list_->setSelectionMode( TreeView::ContiguousSelection );

        mainLayout().addWidget( label_ = new QLabel(this) );

        QMenu* menu( new ContextMenu( &list() ) );
        menu->addMenu( new ColumnSortingMenu( menu, &list() ) );
        menu->addMenu( new ColumnSelectionMenu( menu, &list() ) );

        // close accelerator
        new QShortcut( QKeySequence::Close, this, SLOT(close()) );

    }

    //__________________________________________________
    void CommandDialog::setModel( CommandModel* model )
    {
        list_->setModel( model );
        list_->setOptionName( "ALIEN_COMMAND_LIST" );
        model_ = model;

        connect( model_, SIGNAL(layoutChanged()), SLOT(_updateLabel()) );
    }

    //_____________________________________________________________
    void CommandDialog::_updateLabel()
    {
        if( !model_ ) return;
        const int count( model_->rowCount());
        if( !count ) label_->hide();
        else {
            label_->show();
            if( count == 1 ) label_->setText( tr( " One pending command" ) );
            else label_->setText( tr( " %1 pending commands" ).arg( count ) );
        }

    }

}
