#ifndef AlienCommandList_h
#define AlienCommandList_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "AlienCommand.h"
#include "Counter.h"
#include "Functors.h"

#include <QVector>

namespace Alien
{
    class CommandList final: private Base::Counter<CommandList>
    {
    
        public:
    
        //* constructor
        explicit CommandList():
            Counter( "Alien::CommandList" )
        {}
    
        //* command set (read only)
        const SortedCommand::Set& commands() const
        { return set_; }
    
        //* size
        int size() const
        { return set_.size(); }
    
        //* return true if empty
        bool isEmpty() const
        { return set_.isEmpty(); }
    
        //* true if has pending commands (beside the running one)
        bool hasPendingCommands()
        { return size() > 1; }
    
        //* return front command
        const Command& front() const;
    
        //* append
        /** returns true if appended */
        bool append( const Command& );
    
        //* pop front
        bool removeFirst();
    
        //* abort a given command from list. Return true if aborted
        /** note: the "front" command is never aborted */
        bool abort( const Command& );
    
        private:
    
        using PriorityList = QVector<Command::List>;
    
        //* used to find first non empty
        using NonEmptyFTor = Base::Functor::UnaryFalse<Command::List, &Command::List::isEmpty>;
    
        //* list of commands
        PriorityList list_;
    
        //* set of commands to be kept in sync
        SortedCommand::Set set_;
    
        //* front command
        Command front_;
    
    };
}
#endif
