/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ConfigurationDialog.h"

#include "OptionBrowsedLineEditor.h"
#include "IconNames.h"
#include "IconEngine.h"
#include "OptionCheckBox.h"
#include "OptionColorDisplay.h"
#include "OptionLineEditor.h"
#include "OptionSpinBox.h"
#include "GridLayout.h"

#include <QGroupBox>
#include <QLabel>
#include <QPushButton>

//_________________________________________________________
ConfigurationDialog::ConfigurationDialog( QWidget* parent ):
    BaseConfigurationDialog( parent )
{

    Debug::Throw( "ConfigurationDialog::ConfigurationDialog.\n" );

    // base configuration
    baseConfiguration( 0 );

    QWidget* page;
    QWidget *box;
    GridLayout* gridLayout;

    // commands
    page = &addPage( IconEngine::get( IconNames::PreferencesFileAssociations ), tr( "Applications" ), tr( "Third-party applications" ) );
    {
        box = new QGroupBox( page );
        page->layout()->addWidget( box );

        auto vLayout = new QVBoxLayout;
        vLayout->setMargin(5);
        vLayout->setSpacing(5);
        box->setLayout( vLayout );

        gridLayout = new GridLayout;
        gridLayout->setSpacing(5);
        gridLayout->setMargin(0);
        gridLayout->setMaxCount(2);
        gridLayout->setColumnAlignment( 0, Qt::AlignRight|Qt::AlignVCenter );
        vLayout->addLayout( gridLayout );

        // application finder
        OptionBrowsedLineEditor* editor;
        QLabel* label;
        gridLayout->addWidget( label = new QLabel( tr( "Proxy configuration:" ), box ) );
        gridLayout->addWidget( editor = new OptionBrowsedLineEditor( box, "GSI_PROXY" ) );
        addOptionWidget( editor );
        label->setBuddy( editor );

        editor->setTargetApplication( File("xrdgsiproxy") );

        gridLayout->addWidget( label = new QLabel( tr( "Alien copy:" ), box ) );
        gridLayout->addWidget( editor = new OptionBrowsedLineEditor( box, "ALIEN_CP" ) );
        addOptionWidget( editor );
        label->setBuddy( editor );
        editor->setTargetApplication( File("alien_cp") );

        gridLayout->addWidget( label = new QLabel( tr( "Alien ps:" ), box ) );
        gridLayout->addWidget( editor = new OptionBrowsedLineEditor( box, "ALIEN_PS" ) );
        addOptionWidget( editor );
        label->setBuddy( editor );
        editor->setTargetApplication( File("alien_ps") );

        gridLayout->addWidget( label = new QLabel( tr( "Alien resubmit:"), box ) );
        gridLayout->addWidget( editor = new OptionBrowsedLineEditor( box, "ALIEN_RESUBMIT" ) );
        addOptionWidget( editor );
        label->setBuddy( editor );
        editor->setTargetApplication( File("alien_resubmit") );

        gridLayout->addWidget( label = new QLabel( tr( "Busy box:" ), box ) );
        gridLayout->addWidget( editor = new OptionBrowsedLineEditor( box, "GBBOX" ) );
        addOptionWidget( editor );
        label->setBuddy( editor );
        editor->setTargetApplication( File("gbbox") );

    }

    {
        box = new QGroupBox( tr( "Environment" ), page );
        page->layout()->addWidget( box );
        gridLayout = new GridLayout;
        gridLayout->setSpacing(5);
        gridLayout->setMargin(5);
        gridLayout->setMaxCount(2);
        gridLayout->setColumnAlignment( 0, Qt::AlignRight|Qt::AlignVCenter );
        box->setLayout( gridLayout );

        OptionBrowsedLineEditor* editor;
        QLabel* label;
        gridLayout->addWidget( label = new QLabel( tr( "Certificates:" ), box ) );
        gridLayout->addWidget( editor = new OptionBrowsedLineEditor( box, "CERTIFICATES_PATH" ) );
        editor->setFileMode( QFileDialog::Directory );
        addOptionWidget( editor );
        label->setBuddy( editor );

        OptionLineEditor* hostEditor;
        gridLayout->addWidget( label = new QLabel( tr( "Connection host:" ), box ) );
        gridLayout->addWidget( hostEditor = new OptionLineEditor( box, "ALIEN_HOST" ) );
        addOptionWidget( hostEditor );
        label->setBuddy( hostEditor );

        OptionSpinBox* spinbox;
        gridLayout->addWidget( label = new QLabel( tr( "Connection port:" ), box ) );
        gridLayout->addWidget( spinbox = new OptionSpinBox( box, "ALIEN_PORT" ) );
        spinbox->setMinimum(0);
        spinbox->setMaximum( INT_MAX );
        addOptionWidget( spinbox );
        label->setBuddy( spinbox );
    }

    // appearance
    page = &addPage( IconEngine::get( IconNames::PreferencesAppearance ), tr( "Appearance" ), tr( "Application appearance" ) );
    {
        // running job color
        box = new QGroupBox( tr( "Job State Colors" ), page );
        page->layout()->addWidget( box );

        gridLayout = new GridLayout;
        gridLayout->setSpacing(5);
        gridLayout->setMargin(5);
        gridLayout->setMaxCount(2);
        gridLayout->setColumnAlignment( 0, Qt::AlignRight|Qt::AlignVCenter );
        box->setLayout( gridLayout );

        QLabel* label;
        OptionColorDisplay *display;
        gridLayout->addWidget( label = new QLabel( tr( "Running jobs:" ), box ) );
        gridLayout->addWidget( display = new OptionColorDisplay( box, "RUNNING_JOB_COLOR" ) );
        addOptionWidget( display );
        label->setBuddy( display );

        gridLayout->addWidget( label = new QLabel( tr( "Failed jobs:" ), box ) );
        gridLayout->addWidget( display = new OptionColorDisplay( box, "FAILED_JOB_COLOR" ) );
        addOptionWidget( display );
        label->setBuddy( display );

        gridLayout->addWidget( label = new QLabel( tr( "Completed jobs:" ), box ) );
        gridLayout->addWidget( display = new OptionColorDisplay( box, "COMPLETED_JOB_COLOR" ) );
        addOptionWidget( display );
        label->setBuddy( display );
    }

    {
        // Job log colors
        box = new QGroupBox( tr( "Log Colors" ), page );
        page->layout()->addWidget( box );

        gridLayout = new GridLayout;
        gridLayout->setSpacing(5);
        gridLayout->setMargin(5);
        gridLayout->setMaxCount(2);
        gridLayout->setColumnAlignment( 0, Qt::AlignRight|Qt::AlignVCenter );
        box->setLayout( gridLayout );

        QLabel* label;
        OptionColorDisplay *display;
        gridLayout->addWidget( label = new QLabel(tr(  "Trace information:" ), box ) );
        gridLayout->addWidget( display = new OptionColorDisplay( box, "JOB_TRACE_TRACE_COLOR" ) );
        addOptionWidget( display );
        label->setBuddy( display );

        gridLayout->addWidget( label = new QLabel( tr( "State information:" ), box ) );
        gridLayout->addWidget( display = new OptionColorDisplay( box, "JOB_TRACE_STATE_COLOR" ) );
        addOptionWidget( display );
        label->setBuddy( display );

        gridLayout->addWidget( label = new QLabel( tr( "Process information:" ), box ) );
        gridLayout->addWidget( display = new OptionColorDisplay( box, "JOB_TRACE_PROC_COLOR" ) );
        addOptionWidget( display );
        label->setBuddy( display );

        gridLayout->addWidget( label = new QLabel( tr( "Submission information:" ), box ) );
        gridLayout->addWidget( display = new OptionColorDisplay( box, "JOB_TRACE_SUBMIT_COLOR" ));
        addOptionWidget( display );
        label->setBuddy( display );
    }

    // unsorted
    page = &addPage( IconEngine::get( IconNames::PreferencesUnsorted ), tr( "Unsorted" ), tr( "Additional unsorted settings" ) );
    {
        box = new QWidget( page );
        page->layout()->addWidget( box );

        QVBoxLayout* layout = new QVBoxLayout;
        layout->setMargin(0);
        layout->setSpacing(5);
        box->setLayout( layout );

        QHBoxLayout* hLayout = new QHBoxLayout;
        hLayout->setSpacing(5);
        hLayout->setMargin(0);
        layout->addLayout( hLayout );

        // refresh
        QLabel* label;
        OptionSpinBox* spinbox;
        hLayout->addWidget( label = new QLabel( tr( "Refresh:" ), box ) );
        hLayout->addWidget( spinbox = new OptionSpinBox( box, "REFRESH_RATE" ) );
        spinbox->setToolTip( tr( "Time interval between two refresh" ) );
        spinbox->setSuffix( "s" );
        spinbox->setMinimum( 1 );
        spinbox->setMaximum( 900 );
        addOptionWidget( spinbox );
        label->setBuddy( spinbox );

        OptionCheckBox* checkbox;
        layout->addWidget( checkbox = new OptionCheckBox( tr( "Enable command log" ), box, "LOG_ENABLED" ) );
        addOptionWidget( checkbox );

        layout->addWidget( checkbox = new OptionCheckBox( tr( "Show raw commands in log" ), box, "SHOW_COMMANDS_IN_LOG" ) );
        addOptionWidget( checkbox );

        layout->addWidget( checkbox = new OptionCheckBox( tr( "Hide killed jobs" ), box, "HIDE_KILLED_JOBS" ) );
        addOptionWidget( checkbox );

        layout->addWidget( checkbox = new OptionCheckBox( tr( "Automatically resubmit failed jobs" ), box, "AUTO_RESUBMISSION_ENABLED" ) );
        addOptionWidget( checkbox );

        hLayout = new QHBoxLayout;
        hLayout->setSpacing(5);
        hLayout->setMargin(0);
        layout->addLayout( hLayout );

        hLayout->addSpacing( _checkBoxSpacing() );
        hLayout->addWidget( label = new QLabel( tr( "Maximum number of resubmissions:" ), box ) );
        hLayout->addWidget( spinbox = new OptionSpinBox( box, "MAX_RESUBMISSIONS" ) );
        spinbox->setMinimum( 0 );
        spinbox->setMaximum( 50 );
        spinbox->setSpecialValueText( tr( " Unlimited" ) );
        addOptionWidget( spinbox );
        label->setBuddy( spinbox );

        label->setEnabled( false );
        spinbox->setEnabled( false );
        connect( checkbox, SIGNAL(toggled(bool)), label, SLOT(setEnabled(bool)) );
        connect( checkbox, SIGNAL(toggled(bool)), spinbox, SLOT(setEnabled(bool)) );
    }

    // load initial configuration
    read();

}
