#ifndef DefaultOptions_h
#define DefaultOptions_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "AlienCommandModel.h"
#include "Color.h"
#include "File.h"
#include "Util.h"
#include "XmlOptions.h"

//_____________________________________________________
//* default options installer
void installDefaultOptions()
{

    XmlOptions::get().setAutoDefault( true );

    XmlOptions::get().keep( "APPLICATIONS" );

    XmlOptions::get().keep( "USER_NAME" );
    XmlOptions::get().set( "RUNNING_JOB_COLOR", Base::Color( "#ffcc00" ) );
    XmlOptions::get().set( "FAILED_JOB_COLOR", Base::Color( "#cc0000" ) );
    XmlOptions::get().set( "COMPLETED_JOB_COLOR", Base::Color( "#00aa00" ) );

    XmlOptions::get().set( "JOB_TRACE_TRACE_COLOR", Base::Color( "#00aa00" ) );
    XmlOptions::get().set( "JOB_TRACE_STATE_COLOR", Base::Color( "#aa0000" ) );
    XmlOptions::get().set( "JOB_TRACE_PROC_COLOR", Base::Color( "#0000aa" ) );
    XmlOptions::get().set( "JOB_TRACE_SUBMIT_COLOR", Base::Color( "#ffcc00" ) );

    XmlOptions::get().set<int>( "LIST_ICON_SIZE", 10 );
    XmlOptions::get().set<int>( "REFRESH_RATE", 300 );
    XmlOptions::get().set<bool>( "LOG_ENABLED", false );

    XmlOptions::get().set<bool>( "SHOW_SUMMARY", true );
    XmlOptions::get().set<bool>( "SHOW_COMMAND_LIST", false );
    XmlOptions::get().set<bool>( "SHOW_LOG", false );
    XmlOptions::get().set<bool>( "SHOW_COMMANDS_IN_LOG", false );

    // automatic resubmission
    XmlOptions::get().set<bool>( "AUTO_RESUBMISSION_ENABLED", false );
    XmlOptions::get().set<int>( "MAX_RESUBMISSIONS", 0 );

    // command list
    XmlOptions::get().set<int>( "ALIEN_COMMAND_LIST_MASK", 1<<Alien::CommandModel::Name );
    XmlOptions::get().set<int>( "ALIEN_COMMAND_LIST_SORT_COLUMN", Alien::CommandModel::Priority );

    XmlOptions::get().set<bool>( "HIDE_KILLED_JOBS", true );

    // default open command
    XmlOptions::get().set<bool>( "OPEN_COMMAND", true );

    // alien configuration
    XmlOptions::get().set( "CERTIFICATES_PATH", Util::env( "XrdSecSSLCADIR" ) );
    XmlOptions::get().set( "ALIEN_HOST", "pcapiserv03.cern.ch" );
    XmlOptions::get().set<int>( "ALIEN_PORT", 10000 );

    // resource file
    XmlOptions::get().set( "OLD_RC_FILE", Option( File(".gridwatchrc").addPath( Util::home() ), Option::Flag::None ) );
    XmlOptions::get().set( "RC_FILE", Option( File("gridwatchrc").addPath( Util::config() ), Option::Flag::None ) );

    XmlOptions::get().setAutoDefault( false );

}

#endif
