#ifndef JobManager_h
#define JobManager_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "AlienCommand.h"
#include "JobModel.h"
#include "JobSet.h"
#include "TimeStamp.h"

#include <QWidget>

class TreeView;

//* job manager
class JobManager:public QWidget
{

    //* Qt meta object declaration
    Q_OBJECT

    public:

    //* constructor
    explicit JobManager( QWidget* );

    //* list
    TreeView& list() const
    { return *treeView_; }

    //* process job list
    /** return the list of incomplete jobs, for which a trace command is needed */
    JobSet updateMasterList( JobSet );

    //* process job list
    /** return the list of incomplete jobs, for which a trace command is needed */
    JobSet updateChildrenList( const Job&, JobSet );

    //* process job list
    /** return the list of incomplete jobs, for which a trace command is needed */
    JobSet updateJobList( JobSet );

    //* retrieve selected jobs
    JobSet selectedJobs( bool check = false );

    //* retrieve master jobs
    JobSet masterJobs();

    //* user
    QString user() const
    { return user_; }

    //* update user
    bool setUser( QString );

    //* model
    const JobModel& model() const
    { return model_; }

    //* model
    JobModel& model()
    { return model_; }

    Q_SIGNALS:

    //* busy signal
    void busy( int );

    //* progress
    void progressAvailable( int );

    //* jobs
    void jobListAvailable( QString, const JobSet&, bool = true );

    //* idle
    void idle();

    //* message
    void messageAvailable( const QString& );

    //* request job children
    void childrenListRequested( const Job& );

    public Q_SLOTS:

    //* update specific job
    void processJobInformation( const Job& );

    //* update children
    void processChildrenInformation( const Job&, const JobSet& );

    private Q_SLOTS:

    //* triggered by item expansion
    void _requestChildrenList( const QModelIndex& );

    //* lock childrenList requests
    void _lockRequests()
    { requestsLocked_ = true; }

    //* unlock childrenList requests
    void _unlockRequests()
    { requestsLocked_ = false; }

    private:

    //* update time stamps in job set based on model info
    void _updateTimeStamps( const Job&, JobSet&, bool emitProgress = true );

    //* update time stamps in job set based on model info
    void _updateChildrenStatistics( JobSet& );

    //* update children statistics for master job
    void _updateChildrenStatistics( Job, JobSet );

    //* get list of jobs that need trace for updating timestamps
    JobSet _incompleteJobs( const JobSet& ) const;

    //* display list
    TreeView* treeView_ = nullptr;

    //* current user
    QString user_;

    //* job model
    JobModel model_;

    //* true when no childrenList request should be sent, on item expanded
    bool requestsLocked_ = false;

};

#endif
