#ifndef JobSet_h
#define JobSet_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "Job.h"
#include "JobModel.h"

#include <QDomDocument>
#include <QDomElement>
#include <QSet>

//* set of jobs
class JobSet final
{

    public:

    //* constructor
    explicit JobSet()
    {}

    //* constructor
    explicit JobSet( const QSet<Job>& other ):
        set_( other )
    {}


    //* constructor
    explicit JobSet( QSet<Job>&& other ):
        set_( std::move( other ) )
    {}

    //* construct from dom element
    explicit JobSet( const QDomElement& );

    //*@name accessors
    //@{

    //* dom element
    QDomElement domElement( QDomDocument& ) const;

    //* owner
    QString owner() const
    { return owner_; }

    //* active
    bool isActive() const
    { return active_; }

    //* convert to list
    QList<Job> toList() const
    { return set_.toList(); }

    using const_iterator = QSet<Job>::const_iterator;
    const_iterator begin() const { return set_.begin(); }
    const_iterator end() const { return set_.end(); }
    const_iterator find( const Job& job ) const { return set_.find( job ); }

    int size() const { return set_.size(); }
    bool empty() const { return set_.empty(); }
    bool contains( const Job& job ) const { return set_.contains( job ); }

    //@}

    //*@name modifiers
    //@{

    //* owner
    void setOwner( const QString& owner )
    { owner_ = owner; }

    //* active
    void setActive( bool value )
    { active_ = value; }

    //* assign children flags
    void checkChildren();

    using iterator = QSet<Job>::iterator;
    iterator begin() { return set_.begin(); }
    iterator end() { return set_.end(); }

    const_iterator insert( const Job& job ) { return set_.insert( job ); }
    bool remove( const Job& job ) { return set_.remove( job ); }

    template< class T>
        static JobSet fromList( const T& t )
    { return JobSet( QSet<Job>::fromList(t) ); }

    //@}

    private:

    //* set
    QSet<Job> set_;

    //* active
    bool active_ = false;

    //* owner
    QString owner_;

};

#endif
