/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "JobStatistics.h"

#include "CppUtil.h"
#include "JobSet.h"
#include "XmlDef.h"

#include <algorithm>

//____________________________________________________________
const JobStatistics::FieldNameVector& JobStatistics::_fieldNames()
{
    static const auto fieldNames = Base::makeT<FieldNameVector>(
    {
        QObject::tr( "Total" ),
        QObject::tr( "Completed" ),
        QObject::tr( "Running" ),
        QObject::tr( "Waiting" ),
        QObject::tr( "Failed" ),
        QObject::tr( "Unknown" )
    });
    return fieldNames;
}

//_________________________________________________
JobStatistics::JobStatistics( const QString& user ):
    Counter( "JobStatistics" ),
    time_( TimeStamp::now() ),
    fields_( nFields, 0 ),
    user_( user )
{ Debug::Throw( "JobStatistics::JobStatistics.\n" ); }


//_________________________________________________
JobStatistics::JobStatistics( const JobSet& jobs ):
    Counter( "JobStatistics" ),
    time_( TimeStamp::now() ),
    fields_( nFields, 0 )
{
    set( Total, std::count_if( jobs.begin(), jobs.end(), Job::TotalFTor() ) );
    set( Pending, std::count_if( jobs.begin(), jobs.end(), Job::PendingFTor() ) );
    set( Running, std::count_if( jobs.begin(), jobs.end(), Job::RunningFTor() ) );
    set( Completed, std::count_if( jobs.begin(), jobs.end(), Job::CompletedFTor() ) );
    set( Failed, std::count_if( jobs.begin(), jobs.end(), Job::FailedFTor() ) );
    set( Unknown,  get( Total ) - get( Pending ) - get( Running ) - get( Completed ) - get( Failed ) );
}

//_________________________________________________
bool operator == ( const JobStatistics& first, const JobStatistics& second )
{ return first.fields_ == second.fields_; }
