#ifndef JobStatistics_h
#define JobStatistics_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "Debug.h"
#include "TimeStamp.h"

#include <QString>
#include <QVector>

class JobSet;

//* job statistics container
class JobStatistics final: private Base::Counter<JobStatistics>
{
    public:

    //* allows fast, keyed access to any data field
    enum FieldType
    {
        Total,
        Completed,
        Failed,
        Running,
        Pending,
        Unknown,
        nFields
    };

    //* get field name
    static QString fieldName( int index )
    { return _fieldNames()[index]; }

    //* constructor
    explicit JobStatistics( const QString& user = QString() );

    //* constructor from Job list
    explicit JobStatistics( const JobSet& );

    //*@name accessors
    //@{

    const QString& user() const { return user_; }
    const TimeStamp& time() const { return time_; }
    int get( int type ) const { return fields_[type]; }

    //@}

    //*@name modifiers
    //@{
    void setUser( const QString& value ) { user_ = value; }
    void setTime( const TimeStamp& value ) { time_ = value; }
    void set( FieldType type, int value ) { fields_[type] = value; }
    //@}

    protected:

    //* fields
    using FieldNameVector = QVector<QString>;
    static const FieldNameVector& _fieldNames();

    private:

    //* time stamp
    TimeStamp time_;

    //* fields
    using FieldVector = QVector<int>;
    FieldVector fields_;

    //* user
    QString user_;

    //* equal to operator
    friend bool operator == (const JobStatistics&, const JobStatistics& );

};

#endif
