#ifndef SummaryFrame_h
#define SummaryFrame_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "DockPanel.h"
#include "Job.h"
#include "JobSet.h"
#include "JobStatistics.h"

#include <QLabel>

class GridLayoutItem;
class SummaryBar;

class SummaryFrame: public DockPanel
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! constructor
    explicit SummaryFrame( QWidget* );

    //! summary
    const QString& summaryString() const
    { return summary_; }

    public Q_SLOTS:

    //! update job set
    void updateJobList( QString, const JobSet& jobs, bool updateStamp )
    { _updateJobStatistics( JobStatistics( jobs ), updateStamp ); }

    //! clear
    void clear()
    { _updateJobStatistics( JobStatistics() ); }

    //! update user
    void setUser( const QString& );

    protected:

    //! update labels from integer inputs
    void _updateJobStatistics( const JobStatistics&, bool = true );

    //! set layout item value
    void _set( GridLayoutItem*, int ) const;

    private:

    //! user
    QString userName_;

    //! total jobs label
    GridLayoutItem* totalJobs_;

    //! pending jobs label
    GridLayoutItem* pendingJobs_;

    //! running jobs label
    GridLayoutItem* runningJobs_;

    //! failed jobs label
    GridLayoutItem* failedJobs_;

    //! completed jobs label
    GridLayoutItem* completedJobs_;

    //! unknown jobs label
    GridLayoutItem* unknownJobs_;

    //! last update time
    GridLayoutItem* updated_;

    //! summary bar
    SummaryBar* summaryBar_;

    //! information string
    QString summary_;

};

#endif
